// 
// Copyright (C) 2014, NorthTrader
// V1  Nov-2014  NT  Initial version
// V2  Nov-2014  NT  Added option to display RZs afer TZs too, plus more stats
// V3  Nov-2014  NT  Fixed missing "NinjaScript generatd code" (caused by namespace Description attribute containing concatenated strings)
//
// *** Change version number in Description too!

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// TZPriceAnalysis - analysis of Transient Zones (TZs) by price level
    /// </summary>
	[Description("This version (V3) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides statistics by price level for the given price region. It also includes the option to plot Recurrent Zones on the chart too.\n'RZ' = h-recurrent zone\n'TZ' = h-transient zone\nZones are color-coded depending on which kind of zone they immediately follow at the same price.\nSee the Output window for statistics by price level.")]

	public class TZPriceAnalysis : Indicator
    {
        #region Variables

		//Parameters 
		private int iH_Left = 20;         //hLeft
		private int iH_Right = 20;        //hRight
		private double dPriceLow = 1.2;   //PriceLow
		private double dPriceHigh = 1.4;  //PriceHigh
		private bool bPlotRZAfterTZ = false;  //PlotRZAfterTZ

		//Plot colours 
		private Color colTZAfterRZ = Color.Blue;
		private Color colTZAfterTZ = Color.Red;
		private Color colRZAfterTZ = Color.Green;

		//Other 
		private	int iHRange, iBarMid, iBarFirst, iBarLast;
		private	int iLastP, iTotalBars;
		private double dActualPriceLow, dActualPriceHigh;
		private int[] iRZAfterRZS, iTZAfterRZS, iRZAfterTZS, iTZAfterTZS;
		private double[] dPriceS;
		private bool[] bLastTransientS;
		
        #endregion

		protected override void Initialize()
        {
			//Initialize variables
			iHRange = hLeft + hRight;
			iBarMid = hRight;
			iBarFirst = iBarMid + hLeft;
			iBarLast = 0;  //iBarMid - hLeft;
			iTotalBars = 0;
			dActualPriceLow = dActualPriceHigh = 0;

			DisplayInDataBox	= false;
			PaintPriceMarkers	= false;
            Overlay				= true;
			CalculateOnBarClose = true;	 //Not updated on every tick

        }

		protected override void OnBarUpdate()
        {
			if (CurrentBar == 0) SetVars();
			if (CurrentBar < iHRange) return;

			iTotalBars++;

			//Loop through all prices in region
			for (int liP = 0; liP <= iLastP; liP++)
			{
				
				//Check for price within middle bar
				if (Low[iBarMid] <= dPriceS[liP] && High[iBarMid] >= dPriceS[liP])
				{
					//New zone - check for recurrence
					bool lbTransient = true;
					//Loop from last bar of zone to first
					for (int liBar = iBarLast; liBar <= iBarFirst; liBar++)
					{	
						if (liBar == iBarMid) continue;  //Skip middle bar
						if (Low[liBar] < dPriceS[liP] && High[liBar] >= dPriceS[liP])
						{
							//Recurrent zone
							lbTransient = false;
							break;
						}
					}
					
					Color lcolZ = Color.Empty;
					if (lbTransient)
					{
						if (bLastTransientS[liP])
						{
							iTZAfterTZS[liP]++;
							lcolZ = ColorTZAfterTZ;
						}
						else
						{
							iTZAfterRZS[liP]++;
							lcolZ = ColorTZAfterRZ;
						}
					}
					else
					{
						if (bLastTransientS[liP])
						{
							iRZAfterTZS[liP]++;
							//V2 Display RZs immediately after TZs too (if required by user)
							if (PlotRZAfterTZ) lcolZ = ColorRZAfterTZ;
						}
						else iRZAfterRZS[liP]++;
					}
					if (lcolZ != Color.Empty) DrawRectangle("Z" + CurrentBar + "-" + liP, true, iBarFirst, dPriceS[liP], iBarLast, dPriceS[liP] - TickSize, Color.Empty, lcolZ, 4);
					bLastTransientS[liP] = lbTransient;
				}
			}
			
			//Process and display stats on last bar
			if (CurrentBar == Count - 2) ProcessStats();		
		
		}

        #region Functions

		private void ProcessStats()
		{
			//Called on last bar
			//V2 Display all stats, not just RZs after TZs
			string lsSep = ", ";
			int liTotalZones, liTotalRZs, liTotalTZs, liTotalZonesAfterTZs;
			int liGrandTotalRZsAfterRZs = 0;
			int liGrandTotalRZsAfterTZs = 0;
			int liGrandTotalTZsAfterRZs = 0;
			int liGrandTotalTZsAfterTZs = 0;

			//Output window
			Print(DateTime.Now.ToString());
			Print("h Left = " + hLeft + lsSep + "h Right = " + hRight + lsSep + "Pip = " + TickSize.ToString() + lsSep +
				"Bar = " + BarsPeriod.Value + " " + BarsPeriod.Id);
			Print("Given price region = " + PriceLow.ToString() + " to " + PriceHigh.ToString());
			Print("Total bars = " + iTotalBars);
			for (int liP = 0; liP <= iLastP; liP++)
			{
				liTotalRZs = iRZAfterRZS[liP] + iRZAfterTZS[liP];
				liTotalTZs = iTZAfterRZS[liP] + iTZAfterTZS[liP];
				liTotalZones = liTotalRZs + liTotalTZs;
				if (liTotalZones > 0) 
				{
					//Store first active price level 
					if (dActualPriceLow == 0) dActualPriceLow = dPriceS[liP];
					
					liTotalZonesAfterTZs = iRZAfterTZS[liP] + iTZAfterTZS[liP];
					Print("  Price = " + dPriceS[liP].ToString() + " : " + "TotalZs = " + liTotalZones + lsSep +
						"RZs = " + liTotalRZs + " (" + ((double)liTotalRZs / liTotalZones).ToString("0.0%") + ")" + lsSep +
						"TZs = " + liTotalTZs + " (" + ((double)liTotalTZs / liTotalZones).ToString("0.0%") + ")" +
						((liTotalZonesAfterTZs > 0) ? lsSep + "RZs after TZs = " + ((double)iRZAfterTZS[liP] / liTotalZonesAfterTZs).ToString("0.0%") : ""));
					liGrandTotalRZsAfterRZs += iRZAfterRZS[liP];
					liGrandTotalRZsAfterTZs += iRZAfterTZS[liP];
					liGrandTotalTZsAfterRZs += iTZAfterRZS[liP];
					liGrandTotalTZsAfterTZs += iTZAfterTZS[liP];
				}
				else
				{
					//Store last active price level (assumes no continuous gap)
					if (dActualPriceLow != 0 && dActualPriceHigh == 0) dActualPriceHigh = dPriceS[liP - 1];
				}
			}
			
			if (dActualPriceHigh == 0) dActualPriceHigh = dPriceS[iLastP];
			int liGrandTotalRZs = liGrandTotalRZsAfterRZs + liGrandTotalRZsAfterTZs;
			int liGrandTotalTZs = liGrandTotalTZsAfterRZs + liGrandTotalTZsAfterTZs;
			int liGrandTotalZones = liGrandTotalRZs + liGrandTotalTZs;
			int liGrandTotalZonesAfterRZs = liGrandTotalRZsAfterRZs + liGrandTotalTZsAfterRZs;
			int liGrandTotalZonesAfterTZs = liGrandTotalRZsAfterTZs + liGrandTotalTZsAfterTZs;

			Print("Actual price region = " + dActualPriceLow.ToString() + " to " + dActualPriceHigh.ToString());
			Print("Total zones = " + liGrandTotalZones);
			string lsRZs = liGrandTotalRZs + " (" + ((double)liGrandTotalRZs / liGrandTotalZones).ToString("0.0%") + ")";
			string lsRZsAfterRZs = ((double)liGrandTotalRZsAfterRZs / liGrandTotalZonesAfterRZs).ToString("0.0%") +
				" (" + liGrandTotalRZsAfterRZs + " / " + liGrandTotalZonesAfterRZs + ")";
			string lsRZsAfterTZs = ((double)liGrandTotalRZsAfterTZs / liGrandTotalZonesAfterTZs).ToString("0.0%") +
				" (" + liGrandTotalRZsAfterTZs + " / " + liGrandTotalZonesAfterTZs + ")";
			Print("Total RZs = " + lsRZs + lsSep +
				"Total RZs after RZs = " + lsRZsAfterRZs + lsSep +
				"Total RZs after TZs = " + lsRZsAfterTZs);
			string lsTZs = liGrandTotalTZs + " (" + ((double)liGrandTotalTZs / liGrandTotalZones).ToString("0.0%") + ")";
			string lsTZsAfterRZs = ((double)liGrandTotalTZsAfterRZs / liGrandTotalZonesAfterRZs).ToString("0.0%") +
				" (" + liGrandTotalTZsAfterRZs + " / " + liGrandTotalZonesAfterRZs + ")";
			string lsTZsAfterTZs = ((double)liGrandTotalTZsAfterTZs / liGrandTotalZonesAfterTZs).ToString("0.0%") +
				" (" + liGrandTotalTZsAfterTZs + " / " + liGrandTotalZonesAfterTZs + ")";
			Print("Total TZs = " + lsTZs + lsSep +
				"Total TZs after RZs = " + lsTZsAfterRZs + lsSep +
				"Total TZs after TZs = " + lsTZsAfterTZs);
			
			//Chart
			string lsStats = "h=" + hLeft + "/" + hRight + lsSep;
			lsStats += "Prices=" + dActualPriceLow.ToString() + "-" + dActualPriceHigh.ToString() + lsSep;
			lsStats += "Pip=" + TickSize.ToString() + lsSep;
			lsStats += "Bars=" + iTotalBars + lsSep;
			lsStats += "Zones=" + liGrandTotalZones + lsSep;
			lsStats += "RZs=" + lsRZs + lsSep;
			lsStats += "TZs=" + lsTZs + lsSep;
			lsStats += "RZs after TZs=" + lsRZsAfterTZs;
			DrawTextFixed("stats", lsStats, TextPosition.TopRight);

		}
		
		private void SetVars()
		{
			//Called on first bar because some properties unavailable during Initialize()   
			ClearOutputWindow();
			iLastP = (int)Math.Round((PriceHigh - PriceLow) / TickSize, 0);
			dPriceS = new double[iLastP + 1];
			iRZAfterRZS = new int[iLastP + 1];
			iTZAfterRZS = new int[iLastP + 1];
			iRZAfterTZS = new int[iLastP + 1];
			iTZAfterTZS = new int[iLastP + 1];
			bLastTransientS = new bool[iLastP + 1];
			int liX = 0;
			for (double ldPrice = PriceLow; ldPrice <= PriceHigh; ldPrice += TickSize)
			{
				dPriceS[liX] = Math.Round(ldPrice,4);
				iRZAfterRZS[liX] = iTZAfterRZS[liX] = iRZAfterTZS[liX] = iTZAfterTZS[liX] = 0;
				bLastTransientS[liX] = false;
				liX++;
			}
		}
		
		#endregion

        #region Properties
		
		[Description("Number of bars to the left of middle bar")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("h Left")]
        public int hLeft
        {
            get { return iH_Left; }
            set { iH_Left = Math.Max(1, value); }
        }
		[Description("Number of bars to the right of middle bar")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("h Right")]
        public int hRight
        {
            get { return iH_Right; }
            set { iH_Right = Math.Max(1, value); }
        }
		[Description("Lowest price in analysis area")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("Price Low")]
        public double PriceLow
        {
            get { return dPriceLow; }
            set { dPriceLow = Math.Min(dPriceHigh, value); }
        }
		
		[Description("Highest price in analysis area")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("Price High")]
        public double PriceHigh
        {
            get { return dPriceHigh; }
            set { dPriceHigh = Math.Max(dPriceLow, value); }
        }

		[Description("Show h-recurrent zones that occur immediately after h-transient zones at the same price")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("Show RZ after TZ")]
        public bool PlotRZAfterTZ
        {
            get { return bPlotRZAfterTZ; }
            set { bPlotRZAfterTZ = value; }
        }

		[XmlIgnore]
		[Description("Color of h-transient zone that occurs immediately after a h-recurrent zone at the same price")]
        [GridCategory("Plots")]
        [Gui.Design.DisplayName("Color TZ after RZ")]
		public Color ColorTZAfterRZ
        {
            get { return colTZAfterRZ; }
            set { colTZAfterRZ = value; }
        }
		//Serialize Color object
		[Browsable(false)]
		public string ColorTZAfterRZSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colTZAfterRZ); }
			set { colTZAfterRZ = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore]
		[Description("Color of h-transient zone that occurs immediately after another h-transient zone at the same price")]
        [GridCategory("Plots")]
        [Gui.Design.DisplayName("Color TZ after TZ")]
        public Color ColorTZAfterTZ
        {
            get { return colTZAfterTZ; }
            set { colTZAfterTZ = value; }
        }
		//Serialize Color object
		[Browsable(false)]
		public string ColorTZAfterTZSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colTZAfterTZ); }
			set { colTZAfterTZ = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore]
		[Description("Color of h-recurrent zone that occurs immediately after a h-transient zone at the same price")]
        [GridCategory("Plots")]
        [Gui.Design.DisplayName("Color RZ after TZ")]
		public Color ColorRZAfterTZ
        {
            get { return colRZAfterTZ; }
            set { colRZAfterTZ = value; }
        }
		//Serialize Color object
		[Browsable(false)]
		public string ColorRZAfterTZSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colRZAfterTZ); }
			set { colRZAfterTZ = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TZPriceAnalysis[] cacheTZPriceAnalysis = null;

        private static TZPriceAnalysis checkTZPriceAnalysis = new TZPriceAnalysis();

        /// <summary>
        /// This version (V3) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides statistics by price level for the given price region. It also includes the option to plot Recurrent Zones on the chart too.\n'RZ' = h-recurrent zone\n'TZ' = h-transient zone\nZones are color-coded depending on which kind of zone they immediately follow at the same price.\nSee the Output window for statistics by price level.
        /// </summary>
        /// <returns></returns>
        public TZPriceAnalysis TZPriceAnalysis(Color colorRZAfterTZ, Color colorTZAfterRZ, Color colorTZAfterTZ, int hLeft, int hRight, bool plotRZAfterTZ, double priceHigh, double priceLow)
        {
            return TZPriceAnalysis(Input, colorRZAfterTZ, colorTZAfterRZ, colorTZAfterTZ, hLeft, hRight, plotRZAfterTZ, priceHigh, priceLow);
        }

        /// <summary>
        /// This version (V3) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides statistics by price level for the given price region. It also includes the option to plot Recurrent Zones on the chart too.\n'RZ' = h-recurrent zone\n'TZ' = h-transient zone\nZones are color-coded depending on which kind of zone they immediately follow at the same price.\nSee the Output window for statistics by price level.
        /// </summary>
        /// <returns></returns>
        public TZPriceAnalysis TZPriceAnalysis(Data.IDataSeries input, Color colorRZAfterTZ, Color colorTZAfterRZ, Color colorTZAfterTZ, int hLeft, int hRight, bool plotRZAfterTZ, double priceHigh, double priceLow)
        {
            if (cacheTZPriceAnalysis != null)
                for (int idx = 0; idx < cacheTZPriceAnalysis.Length; idx++)
                    if (cacheTZPriceAnalysis[idx].ColorRZAfterTZ == colorRZAfterTZ && cacheTZPriceAnalysis[idx].ColorTZAfterRZ == colorTZAfterRZ && cacheTZPriceAnalysis[idx].ColorTZAfterTZ == colorTZAfterTZ && cacheTZPriceAnalysis[idx].hLeft == hLeft && cacheTZPriceAnalysis[idx].hRight == hRight && cacheTZPriceAnalysis[idx].PlotRZAfterTZ == plotRZAfterTZ && Math.Abs(cacheTZPriceAnalysis[idx].PriceHigh - priceHigh) <= double.Epsilon && Math.Abs(cacheTZPriceAnalysis[idx].PriceLow - priceLow) <= double.Epsilon && cacheTZPriceAnalysis[idx].EqualsInput(input))
                        return cacheTZPriceAnalysis[idx];

            lock (checkTZPriceAnalysis)
            {
                checkTZPriceAnalysis.ColorRZAfterTZ = colorRZAfterTZ;
                colorRZAfterTZ = checkTZPriceAnalysis.ColorRZAfterTZ;
                checkTZPriceAnalysis.ColorTZAfterRZ = colorTZAfterRZ;
                colorTZAfterRZ = checkTZPriceAnalysis.ColorTZAfterRZ;
                checkTZPriceAnalysis.ColorTZAfterTZ = colorTZAfterTZ;
                colorTZAfterTZ = checkTZPriceAnalysis.ColorTZAfterTZ;
                checkTZPriceAnalysis.hLeft = hLeft;
                hLeft = checkTZPriceAnalysis.hLeft;
                checkTZPriceAnalysis.hRight = hRight;
                hRight = checkTZPriceAnalysis.hRight;
                checkTZPriceAnalysis.PlotRZAfterTZ = plotRZAfterTZ;
                plotRZAfterTZ = checkTZPriceAnalysis.PlotRZAfterTZ;
                checkTZPriceAnalysis.PriceHigh = priceHigh;
                priceHigh = checkTZPriceAnalysis.PriceHigh;
                checkTZPriceAnalysis.PriceLow = priceLow;
                priceLow = checkTZPriceAnalysis.PriceLow;

                if (cacheTZPriceAnalysis != null)
                    for (int idx = 0; idx < cacheTZPriceAnalysis.Length; idx++)
                        if (cacheTZPriceAnalysis[idx].ColorRZAfterTZ == colorRZAfterTZ && cacheTZPriceAnalysis[idx].ColorTZAfterRZ == colorTZAfterRZ && cacheTZPriceAnalysis[idx].ColorTZAfterTZ == colorTZAfterTZ && cacheTZPriceAnalysis[idx].hLeft == hLeft && cacheTZPriceAnalysis[idx].hRight == hRight && cacheTZPriceAnalysis[idx].PlotRZAfterTZ == plotRZAfterTZ && Math.Abs(cacheTZPriceAnalysis[idx].PriceHigh - priceHigh) <= double.Epsilon && Math.Abs(cacheTZPriceAnalysis[idx].PriceLow - priceLow) <= double.Epsilon && cacheTZPriceAnalysis[idx].EqualsInput(input))
                            return cacheTZPriceAnalysis[idx];

                TZPriceAnalysis indicator = new TZPriceAnalysis();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ColorRZAfterTZ = colorRZAfterTZ;
                indicator.ColorTZAfterRZ = colorTZAfterRZ;
                indicator.ColorTZAfterTZ = colorTZAfterTZ;
                indicator.hLeft = hLeft;
                indicator.hRight = hRight;
                indicator.PlotRZAfterTZ = plotRZAfterTZ;
                indicator.PriceHigh = priceHigh;
                indicator.PriceLow = priceLow;
                Indicators.Add(indicator);
                indicator.SetUp();

                TZPriceAnalysis[] tmp = new TZPriceAnalysis[cacheTZPriceAnalysis == null ? 1 : cacheTZPriceAnalysis.Length + 1];
                if (cacheTZPriceAnalysis != null)
                    cacheTZPriceAnalysis.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheTZPriceAnalysis = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// This version (V3) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides statistics by price level for the given price region. It also includes the option to plot Recurrent Zones on the chart too.\n'RZ' = h-recurrent zone\n'TZ' = h-transient zone\nZones are color-coded depending on which kind of zone they immediately follow at the same price.\nSee the Output window for statistics by price level.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TZPriceAnalysis TZPriceAnalysis(Color colorRZAfterTZ, Color colorTZAfterRZ, Color colorTZAfterTZ, int hLeft, int hRight, bool plotRZAfterTZ, double priceHigh, double priceLow)
        {
            return _indicator.TZPriceAnalysis(Input, colorRZAfterTZ, colorTZAfterRZ, colorTZAfterTZ, hLeft, hRight, plotRZAfterTZ, priceHigh, priceLow);
        }

        /// <summary>
        /// This version (V3) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides statistics by price level for the given price region. It also includes the option to plot Recurrent Zones on the chart too.\n'RZ' = h-recurrent zone\n'TZ' = h-transient zone\nZones are color-coded depending on which kind of zone they immediately follow at the same price.\nSee the Output window for statistics by price level.
        /// </summary>
        /// <returns></returns>
        public Indicator.TZPriceAnalysis TZPriceAnalysis(Data.IDataSeries input, Color colorRZAfterTZ, Color colorTZAfterRZ, Color colorTZAfterTZ, int hLeft, int hRight, bool plotRZAfterTZ, double priceHigh, double priceLow)
        {
            return _indicator.TZPriceAnalysis(input, colorRZAfterTZ, colorTZAfterRZ, colorTZAfterTZ, hLeft, hRight, plotRZAfterTZ, priceHigh, priceLow);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// This version (V3) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides statistics by price level for the given price region. It also includes the option to plot Recurrent Zones on the chart too.\n'RZ' = h-recurrent zone\n'TZ' = h-transient zone\nZones are color-coded depending on which kind of zone they immediately follow at the same price.\nSee the Output window for statistics by price level.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TZPriceAnalysis TZPriceAnalysis(Color colorRZAfterTZ, Color colorTZAfterRZ, Color colorTZAfterTZ, int hLeft, int hRight, bool plotRZAfterTZ, double priceHigh, double priceLow)
        {
            return _indicator.TZPriceAnalysis(Input, colorRZAfterTZ, colorTZAfterRZ, colorTZAfterTZ, hLeft, hRight, plotRZAfterTZ, priceHigh, priceLow);
        }

        /// <summary>
        /// This version (V3) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides statistics by price level for the given price region. It also includes the option to plot Recurrent Zones on the chart too.\n'RZ' = h-recurrent zone\n'TZ' = h-transient zone\nZones are color-coded depending on which kind of zone they immediately follow at the same price.\nSee the Output window for statistics by price level.
        /// </summary>
        /// <returns></returns>
        public Indicator.TZPriceAnalysis TZPriceAnalysis(Data.IDataSeries input, Color colorRZAfterTZ, Color colorTZAfterRZ, Color colorTZAfterTZ, int hLeft, int hRight, bool plotRZAfterTZ, double priceHigh, double priceLow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TZPriceAnalysis(input, colorRZAfterTZ, colorTZAfterRZ, colorTZAfterTZ, hLeft, hRight, plotRZAfterTZ, priceHigh, priceLow);
        }
    }
}
#endregion
